
class SVG():
    '''Klasse zum Schreiben von SVG-Vektorgrafiken'''

    def __init__(self, width=600, height=400):
        '''Erzeugt ein Bild der Grösse width x height'''
        self.width = width
        self.height = height
        self.stroke = 'black'
        self.strokewidth = 1
        self.fill = 'none'
        self.code = ''

    def write(self, dateiname):
        '''Schreibt die Bilddaten in die Datei dateiname'''
        txt  = '<?xml version="1.0" encoding="utf-8"?>\n'
        txt += '<svg xmlns="http://www.w3.org/2000/svg" '
        txt += 'width="{0}" '.format(self.width)
        txt += 'height="{0}">\n\n'.format(self.height)
        txt += self.code
        txt += '</svg>\n'
        
        fd = open(dateiname, mode='w')
        fd.write(txt)
        fd.close()

    def line(self, P1, P2):
        '''Verbindet die Punkte P1=[x1,y1] und P2=[x2,y2] durch
        eine Strecke.'''
        txt  = '<line x1="{0[0]}" y1="{0[1]}" '.format(P1)
        txt += 'x2="{0[0]}" y2="{0[1]}" '.format(P2)   
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += '/>\n\n'
        self.code += txt

    def circle(self, M, r):
        '''Zeichnet einen Kreis mit Mittelpunkt M=[x,y] und Radius r'''
        txt  = '<circle '
        txt += 'cx="{0}" '.format(M[0])
        txt += 'cy="{0}" '.format(M[1])
        txt += 'r="{0}" '.format(r)   
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += 'fill="{0}" '.format(self.fill)
        txt += '/>\n\n'
        self.code += txt

    def rect(self, P, w, h):
        '''Zeichnet ein Rechteck mit linker oberer Ecke P=[x,y],
        der Breite b und der Höhe h.'''
        txt  = '<rect '
        txt += 'x="{0}" '.format(P[0])
        txt += 'y="{0}" '.format(P[1])
        txt += 'width="{0}" '.format(w)   
        txt += 'height="{0}" '.format(h)   
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += 'fill="{0}" '.format(self.fill)
        txt += '/>\n\n'
        self.code += txt

    def ellipse(self, M, a, b):
        '''Zeichnet eine Ellipse mit Mittelpunkt M=[x,y] und den
        Halbachsen a und b.'''
        txt  = '<ellipse '
        txt += 'cx="{0}" '.format(M[0])
        txt += 'cy="{0}" '.format(M[1])
        txt += 'rx="{0}" '.format(a)   
        txt += 'ry="{0}" '.format(b)   
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += 'fill="{0}" '.format(self.fill)
        txt += '/>\n\n'
        self.code += txt

    def polyline(self, pointlist):
        '''Verbindet die Punkte [x1,y1], ..., [xn,yn] in pointlist'''
        txt = '<polyline points="\n'
        for P in pointlist:
            txt += '{0[0]},{0[1]}\n'.format(P)
        txt += '"\n'
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += 'fill="{0}" '.format(self.fill)
        txt += '/>\n\n'
        self.code += txt

    def polygon(self, pointlist):
        '''Verbindet die Punkte [x1,y1], ..., [xn,yn] in pointlist
        durch ein geschlossenes Polygon.''' 
        txt = '<polygon points="\n'
        for P in pointlist:
            txt += '{0[0]},{0[1]}\n'.format(P)
        txt += '"\n'
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += 'fill="{0}" '.format(self.fill)
        txt += '/>\n\n'
        self.code += txt

    def cubic_spline(self, pointlist):
        '''Zeichnet eine kubische Spline durch die Punkte [x1,y1],
        ..., [xn, yn] der Punktliste pointlist'''
        txt  = '<path '
        txt += 'd="'
        txt += 'M {0[0]},{0[1]}\n'.format(pointlist[0])
        txt += 'C '
        for i in range(1, len(pointlist)):
            txt += '{0[0]},{0[1]}\n'.format(pointlist[i])
        txt += '"\n'
        txt += 'fill="{0}" '.format(self.fill)
        txt += 'stroke-width="{0}" '.format(self.strokewidth)
        txt += 'stroke="{0}" '.format(self.stroke)
        txt += '/>\n\n'
        self.code += txt


if __name__ == "__main__":

    img = SVG(600, 400)
    img.line([50, 50], [100, 100])

    img.stroke="blue"
    img.strokewidth=5
    img.fill="yellow"
    img.circle([200,100], 40)

    img.stroke="red"
    img.strokewidth=10
    img.fill="cyan"
    img.rect([300,60], 100, 80)

    img.stroke="#aaaaaa"
    img.strokewidth=10
    img.fill="#333333"
    img.ellipse([500,100], 60, 40)

    img.stroke="#31f87b"
    img.strokewidth=5
    img.fill="#8a04c7"
    img.polygon([[50,200],[100,300],[150,200],[200,300]])
    
    img.stroke="#6c9228"
    img.strokewidth=5
    img.fill="none"
    img.polyline([[300,200],[400,300],[500,200],[600,300]])

    img.stroke="#c3a411"
    img.strokewidth=5
    img.fill="none"
    img.cubic_spline([[300,200],[400,300],[500,200],[600,300]])    

    img.write('svg-test.svg')
        



